__doc__ = """
    arguments :
        --config    path of the configuration otherwise it will look for the default in the working directory
"""
from flask import Flask,render_template,send_from_directory,request, redirect, Response, session
import flask
import transport
from transport import providers
import cms
from cms import Plugin
import sys
import os
import json
import copy
import io
import base64
from jinja2 import Environment, BaseLoader
import typer
from typing_extensions import Annotated
from typing import Optional


import pandas as pd
import uuid
import datetime
import requests

from cms import disk, cloud, engine
import cms.sites

_app = Flask(__name__)
cli = typer.Typer()

def _getHandler (app_id,resource=None) :
    global _qcms
    _id = _getId(app_id,resource)
    
    return _qcms._apps[_id]
def _getId(app_id,app_x):
    if app_x not in [None,''] :
        _uri =  '/'.join([app_id,app_x])
        return _uri[:-1] if _uri.endswith('/') else _uri
    return app_id
def _setHandler (app_id,resource) :
    session['app_id'] = _getId(app_id,resource)

@_app.route("/<_id>/robots.txt")
@_app.route("/robots.txt",defaults={'_id':None})
def robots_txt(_id):
    """
    This function will generate a robots expression for a variety of crawlers, the paths will be provided by
    menu options
    """
    global _qcms
    _site = _qcms.get (_id)
    _routes = _site.get('system.routes')
    _context = _site.get('system.context')
    _info  = [f'''
    User-agent: *
    Allow: /{_context}
    ''']

    if _routes :
        for _key in _routes:
            _uri = '/'.join(["",_context,_key])
            _info.append(f'''
    User-agent: *
    Allow: {_uri}
                        ''')
    
    # return '\n'.join(_info),200,{'Content-Type':'plain/text'}
    return Response('\n'.join(_info), mimetype='text/plain')  
def _getIndex (app_id ,resource=None):
    _handler = _getHandler(app_id,resource)
    _layout = _handler.layout()
    _status_code = 200
    global _qcms
    _index_page = 'index.html'
    _args = {}
    
    try :
        _uri = os.sep.join([_layout['root'],_layout['index']])
        _id = _getId(app_id,resource)
        _args = _qcms.render(_uri,'index',_id)
        
    except Exception as e:
        _status_code = 404
        _index_page = '404.html'
        print(e)
    return render_template(_index_page,**_args),_status_code
@_app.route("/")
def _index ():
    # return _getIndex('main')
    global _qcms
    _app = _qcms.get(None)
    _uri = os.sep.join([_app.get('layout.root'),_app.get('layout.index')])
    _html = _qcms.render(_uri,'index')
    return  render_template('index.html',**_qcms.render(_uri,'index')),200

@_app.route("/<app>/<resource>")
@_app.route("/<app>",defaults={'resource':None})
def _altIndex(app,resource):
    global _qcms
    _id = _getId(app,resource)
    _site = _qcms.get(_id)
    _uri = os.sep.join([_site.get('layout.root'),_site.get('layout.index')])
    return  render_template('index.html',**_qcms.render(_uri,'index',_id)),200

@_app.route('/<app>/dialog')  
@_app.route('/dialog',defaults={'app':None})
def _getdialog(app):
    global _qcms
    _site = _qcms.get(app)
    _uri = request.headers['uri']    
    _id = request.headers['dom']
    # _html = ''.join(["<div style='padding:1%'>",str( e.render(**_args)),'</div>'])
    _args = _qcms.render(_uri,'html',app) #session.get('app_id','main'))
    _args['title'] = _id
    return render_template('dialog.html',**_args) #title=_id,html=_html)

@_app.route("/api/<module>/<name>",defaults={'app':None,'key':None},methods=['GET','POST','DELETE','PUT'])
@_app.route("/<app>/api/<module>/<name>",defaults={'key':None},methods=['GET','POST','DELETE','PUT'])
@_app.route("/<app>/<key>/api/<module>/<name>",methods=['GET','POST','DELETE','PUT'])
def _proxyCall(app,key,module,name):
    global _qcms
    
    _id = _getId(app,key)
    
    _site = _qcms.get(_id)
    _uri = f'api/{module}/{name}'
    _delegate = cms.delegate()
    return _delegate(uri=_uri, handler=_site, request=request)

@_app.route('/version')
def _version ():
    global _qcms
    _site = _qcms.get()
    global _config 
    return _site.get('system.version')
@_app.route("/reload/<key>")
def _reload(key) :
    global _qcms

    _site = _qcms.get(None)
    _systemKey = _site.get('system.source.key')
    if _systemKey == key and _systemKey:
        _site.reload()
        return "",200
    else:
        return "",403

@_app.route('/reload',methods=['POST'])
def reload():
    _key = request.headers['key'] if 'key' in request.headers else None
    return _reload(_key)
 

@_app.route('/page',methods=['POST'],defaults={'app_id':None,'key':None})
@_app.route('/<app_id>/page',methods=['POST'],defaults={'key':None})
@_app.route('/<app_id>/<key>/page',methods=['POST'])
def getPostedPage(app_id,key):
    """
    This function will return a page given a location parameter
    """
    global _qcms
    _id = _getId(app_id,key)
    _site = _qcms.get(_id)
    _uri = request.headers['uri']
    if 'dom' not in request.headers :
        _id = _uri.split('/')[-1].split('.')[0]
    else:
        _id = request.headers['dom']

    if 'read?uri=' in _uri or 'download?doc=' in _uri :
        _uri = _uri.split('=')[1]
    _args = _qcms.render(_uri,_id,_getId(app_id,key)) #session.get(app_id,'main'))
    return _args[_id],200

@_app.route('/page',defaults={'app_id':None,'resource':None})
@_app.route('/<app_id>/page',defaults={'resource':None},methods=['GET'])
@_app.route('/<app_id>/<resource>/page',methods=['GET'])
def _cms_page (app_id,resource):
    global _qcms
    _uri = request.args['uri']
    _title = request.args['title'] if 'title' in request.args else ''
    _args = _qcms.render(_uri,_title,app_id)
    return _args[_title],200


@cli.command()
def start (
    path:Annotated[str,typer.Argument(help="path of the manifest file")]='qcms-manifest.json',
    port:int=typer.Option(default=None, help="port number to serve on (will override configuration)")

    ) :
    """
    This function is designed to start the application with its associated manifest (configuration) location
    :path   path to the  the manifest
    :shared run in shared mode i.e 
    """
    global _qcms

    if os.path.exists(path) and os.path.isfile(path):
        _args = {'path':path}
        _args['location'] = path
        _args['shared'] = True
        
        _qcms = cms.sites.QCMS(**_args)
        _args = _qcms.get().get('system.app')
        
        if port :
            _args['port'] = port
        _app.secret_key = str(uuid.uuid4())
        _app.config['PERMANENT_SESSION_LIFETIME'] = datetime.timedelta(hours=24)
        _app.run(**_args)
        _status = 'found'
    else:
        _status = 'not found'
    print(f'''
            manifest: {path}
            status  : {_status}
        ''')
@cli.command(name='help')
def _help() :
    pass
if __name__ == '__main__' :
    cli()
